=== modified file 'NEWS'
--- NEWS	2006-07-11 17:12:14 +0000
+++ NEWS	2006-07-12 14:24:06 +0000
@@ -2,6 +2,11 @@
 
   IMPROVEMENTS:
 
+   * The revision specifier "revno:" is extended to accept the syntax
+     revno:N:branch. For example,
+     revno:42:http://bazaar-vcs.org/bzr/bzr.dev/ means revision 42 in
+     bzr.dev.  (Matthieu Moy)
+
    * No default ignore rules are applied by bzr - only the rules in .bzrignore
      are considered. This fixes bugs with default rules being enforced no
      matter what. The old list of ignore rules from bzr is available by
@@ -83,6 +88,10 @@
      Also updates things like 'http+pycurl://' if pycurl is not present.
      (John Arbash Meinel) (Malone #47821, #52204)
 
+   * New env variable BZR_PROGRESS_BAR, sets the default progress bar type.
+     Can be set to 'none' or 'dummy' to disable the progress bar, 'dots' or 
+     'tty' to create the respective type. (John Arbash Meinel, #42197, #51107)
+
   BUG FIXES:
 
     * Fix shadowed definition of TestLocationConfig that caused some 
@@ -113,6 +122,29 @@
     * File-ids containing single- or double-quotes are handled correctly by
       push.  (#52227, Aaron Bentley)
 
+    * Normalize unicode filenames to ensure cross-platform consistency.
+      (John Arbash Meinel, #43689)
+
+    * The argument parser can now handle '-' as an argument. Currently
+      no code interprets it specially (it is mostly handled as a file named 
+      '-'). But plugins, and future operations can use it.
+      (John Arbash meinel, #50984)
+
+    * Bundles can properly read binary files with a plain '\r' in them.
+      (John Arbash Meinel, #51927)
+
+    * Tuning iter_entries() to be more efficient (John Arbash Meinel, #5444)
+
+    * Lots of win32 fixes (the test suite passes again).
+      (John Arbash Meinel, #50155)
+
+    * Handle openbsd returning None for sys.getfilesystemencoding() (#41183) 
+
+    * Support ftp APPE (append) to allow Knits to be used over ftp (#42592)
+
+    * Removals are only committed if they match the filespec (or if there is
+      no filespec).  (#46635, Aaron Bentley)
+
     * smart-add recurses through all supplied directories 
       (John Arbash Meinel, #52578)
 

=== modified file 'bzrlib/add.py'
--- bzrlib/add.py	2006-07-11 17:12:14 +0000
+++ bzrlib/add.py	2006-07-12 14:24:06 +0000
@@ -273,9 +273,7 @@
                     ignore_glob = tree.is_ignored(subp)
                     if ignore_glob is not None:
                         # mutter("skip ignored sub-file %r", subp)
-                        if ignore_glob not in ignored:
-                            ignored[ignore_glob] = []
-                        ignored[ignore_glob].append(subp)
+                        ignored.setdefault(ignore_glob, []).append(subp)
                     else:
                         #mutter("queue to add sub-file %r", subp)
                         dirs_to_add.append((FastPath(subp, subf), this_ie))

=== modified file 'bzrlib/benchmarks/__init__.py'
--- bzrlib/benchmarks/__init__.py	2006-07-06 02:27:26 +0000
+++ bzrlib/benchmarks/__init__.py	2006-07-11 14:47:00 +0000
@@ -57,7 +57,7 @@
         
         No files change are included.
         """
-        tree = BzrDir.create_standalone_workingtree(directory_name)
+        tree = bzrdir.BzrDir.create_standalone_workingtree(directory_name)
         tree.lock_write()
         tree.branch.lock_write()
         tree.branch.repository.lock_write()
@@ -82,7 +82,7 @@
         commit in its revision-history is a merge another commit in the other
         tree.
         """
-        tree = BzrDir.create_standalone_workingtree(directory_name)
+        tree = bzrdir.BzrDir.create_standalone_workingtree(directory_name)
         tree.lock_write()
         try:
             tree2 = tree.bzrdir.sprout('tree2').open_workingtree()

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-07-04 15:04:11 +0000
+++ bzrlib/builtins.py	2006-07-12 12:36:57 +0000
@@ -375,6 +375,9 @@
     encoding_type = 'replace'
 
     def run(self, names_list):
+        if names_list is None:
+            names_list = []
+
         if len(names_list) < 2:
             raise BzrCommandError("missing file argument")
         tree, rel_names = tree_files(names_list)
@@ -1139,6 +1142,10 @@
                 # FIXME diff those two files. rbc 20051123
                 raise BzrCommandError("Files are in different branches")
             file_list = None
+        except NotBranchError:
+            # Don't raise an error when bzr diff is called from
+            # outside a working tree.
+            tree1, tree2 = None, None
         if revision is not None:
             if tree2 is not None:
                 raise BzrCommandError("Can't specify -r with two branches")

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2006-07-01 19:03:33 +0000
+++ bzrlib/commands.py	2006-07-11 21:46:11 +0000
@@ -368,6 +368,9 @@
             if argsover:
                 args.append(a)
                 continue
+            elif a == '-':
+                args.append(a)
+                continue
             elif a == '--':
                 # We've received a standalone -- No more flags
                 argsover = True

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2006-06-21 21:35:06 +0000
+++ bzrlib/commit.py	2006-07-11 15:04:57 +0000
@@ -497,6 +497,10 @@
         None; inventory entries that are carried over untouched have their
         revision set to their prior value.
         """
+        # ESEPARATIONOFCONCERNS: this function is diffing and using the diff
+        # results to create a new inventory at the same time, which results
+        # in bugs like #46635.  Any reason not to use/enhance compare_trees?
+        # ADHB 11-07-2006
         mutter("Selecting files for commit with filter %s", self.specific_files)
         # iter_entries does not visit the ROOT_ID node so we need to call
         # self._emit_progress_update once by hand.
@@ -534,6 +538,20 @@
             else:
                 self.reporter.snapshot_change(change, path)
 
+        if not self.specific_files:
+            return
+
+        # ignore removals that don't match filespec
+        for path, new_ie in self.basis_inv.iter_entries():
+            if new_ie.file_id in self.work_inv:
+                continue
+            if is_inside_any(self.specific_files, path):
+                continue
+            ie = new_ie.copy()
+            ie.revision = None
+            self.builder.record_entry_contents(ie, self.parent_invs, path,
+                                               self.basis_tree)
+
     def _emit_progress_update(self):
         """Emit an update to the progress bar."""
         self.pb.update("Committing", self.pb_count, self.pb_total)

=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2006-07-05 05:18:06 +0000
+++ bzrlib/diff.py	2006-07-12 12:36:57 +0000
@@ -256,10 +256,14 @@
     The more general form is show_diff_trees(), where the caller
     supplies any two trees.
     """
-    output = sys.stdout
     def spec_tree(spec):
-        revision_id = spec.in_store(tree.branch).rev_id
-        return tree.branch.repository.revision_tree(revision_id)
+        if tree:
+            revision = spec.in_store(tree.branch)
+        else:
+            revision = spec.in_store(None)
+        revision_id = revision.rev_id
+        branch = revision.branch
+        return branch.repository.revision_tree(revision_id)
     if old_revision_spec is None:
         old_tree = tree.basis_tree()
     else:

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-07-10 02:16:27 +0000
+++ bzrlib/errors.py	2006-07-12 02:20:47 +0000
@@ -1,15 +1,15 @@
 # Copyright (C) 2005, 2006 Canonical
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -280,6 +280,10 @@
             self.extra = ''
 
 
+class InvalidNormalization(PathError):
+    """Path %(path)r is not unicode normalized"""
+
+
 # TODO: This is given a URL; we try to unescape it but doing that from inside
 # the exception object is a bit undesirable.
 # TODO: Probably this behavior of should be a common superclass 
@@ -966,6 +970,14 @@
     """A nested progress bar was not 'finished' correctly."""
 
 
+class InvalidProgressBarType(BzrNewError):
+    """Environment variable BZR_PROGRESS_BAR='%(bar_type)s is not a supported type
+Select one of: %(valid_types)s"""
+
+    def __init__(self, bar_type, valid_types):
+        BzrNewError.__init__(self, bar_type=bar_type, valid_types=valid_types)
+
+
 class UnsupportedOperation(BzrNewError):
     """The method %(mname)s is not supported on objects of type %(tname)s."""
     def __init__(self, method, method_self):

=== modified file 'bzrlib/hashcache.py'
--- bzrlib/hashcache.py	2006-03-20 18:34:33 +0000
+++ bzrlib/hashcache.py	2006-07-12 05:22:48 +0000
@@ -42,20 +42,6 @@
 FP_CTIME_COLUMN = 2
 FP_MODE_COLUMN = 5
 
-def _fingerprint(abspath):
-    try:
-        fs = os.lstat(abspath)
-    except OSError:
-        # might be missing, etc
-        return None
-
-    if stat.S_ISDIR(fs.st_mode):
-        return None
-
-    # we discard any high precision because it's not reliable; perhaps we
-    # could do better on some systems?
-    return (fs.st_size, long(fs.st_mtime),
-            long(fs.st_ctime), fs.st_ino, fs.st_dev, fs.st_mode)
 
 
 class HashCache(object):
@@ -131,7 +117,7 @@
         
         for inum, path, cache_entry in prep:
             abspath = pathjoin(self.root, path)
-            fp = _fingerprint(abspath)
+            fp = self._fingerprint(abspath)
             self.stat_count += 1
             
             cache_fp = cache_entry[1]
@@ -142,13 +128,12 @@
                 self.needs_write = True
                 del self._cache[path]
 
-
     def get_sha1(self, path):
         """Return the sha1 of a file.
         """
         abspath = pathjoin(self.root, path)
         self.stat_count += 1
-        file_fp = _fingerprint(abspath)
+        file_fp = self._fingerprint(abspath)
         
         if not file_fp:
             # not a regular file or not existing
@@ -164,22 +149,26 @@
             cache_sha1, cache_fp = None, None
 
         if cache_fp == file_fp:
+            ## mutter("hashcache hit for %s %r -> %s", path, file_fp, cache_sha1)
+            ## mutter("now = %s", time.time())
             self.hit_count += 1
             return cache_sha1
         
         self.miss_count += 1
 
-
         mode = file_fp[FP_MODE_COLUMN]
         if stat.S_ISREG(mode):
-            digest = sha_file(file(abspath, 'rb', buffering=65000))
+            digest = self._really_sha1_file(abspath)
         elif stat.S_ISLNK(mode):
             digest = sha.new(os.readlink(abspath)).hexdigest()
         else:
             raise BzrError("file %r: unknown file stat mode: %o"%(abspath,mode))
 
-        now = int(time.time())
-        if file_fp[FP_MTIME_COLUMN] >= now or file_fp[FP_CTIME_COLUMN] >= now:
+        # window of 3 seconds to allow for 2s resolution on windows,
+        # unsynchronized file servers, etc.
+        cutoff = self._cutoff_time()
+        if file_fp[FP_MTIME_COLUMN] >= cutoff \
+                or file_fp[FP_CTIME_COLUMN] >= cutoff:
             # changed too recently; can't be cached.  we can
             # return the result and it could possibly be cached
             # next time.
@@ -191,16 +180,24 @@
             # need to let sufficient time elapse before we may cache this entry
             # again.  If we didn't do this, then, for example, a very quick 1
             # byte replacement in the file might go undetected.
-            self.danger_count += 1 
+            ## mutter('%r modified too recently; not caching', path)
+            self.danger_count += 1
             if cache_fp:
                 self.removed_count += 1
                 self.needs_write = True
                 del self._cache[path]
         else:
+            ## mutter('%r added to cache: now=%f, mtime=%d, ctime=%d',
+            ##        path, time.time(), file_fp[FP_MTIME_COLUMN],
+            ##        file_fp[FP_CTIME_COLUMN])
             self.update_count += 1
             self.needs_write = True
             self._cache[path] = (digest, file_fp)
         return digest
+
+    def _really_sha1_file(self, abspath):
+        """Calculate the SHA1 of a file by reading the full text"""
+        return sha_file(file(abspath, 'rb', buffering=65000))
         
     def write(self):
         """Write contents of cache to file."""
@@ -218,10 +215,10 @@
                 print >>outf
             outf.commit()
             self.needs_write = False
-            mutter("write hash cache: %s hits=%d misses=%d stat=%d recent=%d updates=%d",
-                   self.cache_file_name(), self.hit_count, self.miss_count,
-                   self.stat_count,
-                   self.danger_count, self.update_count)
+            ## mutter("write hash cache: %s hits=%d misses=%d stat=%d recent=%d updates=%d",
+            ##        self.cache_file_name(), self.hit_count, self.miss_count,
+            ##        self.stat_count,
+            ##        self.danger_count, self.update_count)
         finally:
             if not outf.closed:
                 outf.abort()
@@ -244,7 +241,6 @@
             self.needs_write = True
             return
 
-
         hdr = inf.readline()
         if hdr != CACHE_HEADER:
             mutter('cache header marker not found at top of %s;'
@@ -275,7 +271,24 @@
             self._cache[path] = (sha1, fp)
 
         self.needs_write = False
+
+    def _cutoff_time(self):
+        """Return cutoff time.
+
+        Files modified more recently than this time are at risk of being
+        undetectably modified and so can't be cached.
+        """
+        return int(time.time()) - 3
            
-
-
-        
+    def _fingerprint(self, abspath):
+        try:
+            fs = os.lstat(abspath)
+        except OSError:
+            # might be missing, etc
+            return None
+        if stat.S_ISDIR(fs.st_mode):
+            return None
+        # we discard any high precision because it's not reliable; perhaps we
+        # could do better on some systems?
+        return (fs.st_size, long(fs.st_mtime),
+                long(fs.st_ctime), fs.st_ino, fs.st_dev, fs.st_mode)

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2006-06-26 19:06:55 +0000
+++ bzrlib/inventory.py	2006-07-02 05:45:50 +0000
@@ -36,6 +36,7 @@
 import types
 
 import bzrlib
+from bzrlib import errors, osutils
 from bzrlib.osutils import (pumpfile, quotefn, splitpath, joinpath,
                             pathjoin, sha_strings)
 from bzrlib.errors import (NotVersionedError, InvalidEntryName,
@@ -1061,7 +1062,7 @@
 
         Returns the new entry object."""
         
-        parts = bzrlib.osutils.splitpath(relpath)
+        parts = osutils.splitpath(relpath)
 
         if len(parts) == 0:
             if file_id is None:
@@ -1237,6 +1238,16 @@
     """
     if file_id is None:
         file_id = bzrlib.workingtree.gen_file_id(name)
+
+    norm_name, can_access = osutils.normalized_filename(name)
+    if norm_name != name:
+        if can_access:
+            name = norm_name
+        else:
+            # TODO: jam 20060701 This would probably be more useful
+            #       if the error was raised with the full path
+            raise errors.InvalidNormalization(name)
+
     if kind == 'directory':
         return InventoryDirectory(file_id, name, parent_id)
     elif kind == 'file':
@@ -1247,7 +1258,6 @@
         raise BzrError("unknown kind %r" % kind)
 
 
-
 _NAME_RE = None
 
 def is_valid_name(name):

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2006-06-30 22:42:39 +0000
+++ bzrlib/osutils.py	2006-07-03 18:51:47 +0000
@@ -264,14 +264,19 @@
     try:
         fancy_rename(old, new, rename_func=os.rename, unlink_func=os.unlink)
     except OSError, e:
-        if e.errno in (errno.EPERM, errno.EACCES, errno.EBUSY):
-            # If we try to rename a non-existant file onto cwd, we get EPERM
-            # instead of ENOENT, this will raise ENOENT if the old path
-            # doesn't exist
+        if e.errno in (errno.EPERM, errno.EACCES, errno.EBUSY, errno.EINVAL):
+            # If we try to rename a non-existant file onto cwd, we get 
+            # EPERM or EACCES instead of ENOENT, this will raise ENOENT 
+            # if the old path doesn't exist, sometimes we get EACCES
+            # On Linux, we seem to get EBUSY, on Mac we get EINVAL
             os.lstat(old)
         raise
 
 
+def _mac_getcwd():
+    return unicodedata.normalize('NFKC', os.getcwdu())
+
+
 # Default is to just use the python builtins, but these can be rebound on
 # particular platforms.
 abspath = _posix_abspath
@@ -315,6 +320,8 @@
     def rmtree(path, ignore_errors=False, onerror=_win32_delete_readonly):
         """Replacer for shutil.rmtree: could remove readonly dirs/files"""
         return shutil.rmtree(path, ignore_errors, onerror)
+elif sys.platform == 'darwin':
+    getcwd = _mac_getcwd
 
 
 def get_terminal_encoding():
@@ -788,39 +795,35 @@
     return _platform_normalizes_filenames
 
 
+def _accessible_normalized_filename(path):
+    """Get the unicode normalized path, and if you can access the file.
+
+    On platforms where the system normalizes filenames (Mac OSX),
+    you can access a file by any path which will normalize correctly.
+    On platforms where the system does not normalize filenames 
+    (Windows, Linux), you have to access a file by its exact path.
+
+    Internally, bzr only supports NFC/NFKC normalization, since that is 
+    the standard for XML documents.
+
+    So return the normalized path, and a flag indicating if the file
+    can be accessed by that path.
+    """
+
+    return unicodedata.normalize('NFKC', unicode(path)), True
+
+
+def _inaccessible_normalized_filename(path):
+    __doc__ = _accessible_normalized_filename.__doc__
+
+    normalized = unicodedata.normalize('NFKC', unicode(path))
+    return normalized, normalized == path
+
+
 if _platform_normalizes_filenames:
-    def unicode_filename(path):
-        """Make sure 'path' is a properly normalized filename.
-
-        On platforms where the system normalizes filenames (Mac OSX),
-        you can access a file by any path which will normalize
-        correctly.
-        Internally, bzr only supports NFC/NFKC normalization, since
-        that is the standard for XML documents.
-        So we return an normalized path, and indicate this has been
-        properly normalized.
-
-        :return: (path, is_normalized) Return a path which can
-                access the file, and whether or not this path is
-                normalized.
-        """
-        return unicodedata.normalize('NFKC', path), True
+    normalized_filename = _accessible_normalized_filename
 else:
-    def unicode_filename(path):
-        """Make sure 'path' is a properly normalized filename.
-
-        On platforms where the system does not normalize filenames 
-        (Windows, Linux), you have to access a file by its exact path.
-        Internally, bzr only supports NFC/NFKC normalization, since
-        that is the standard for XML documents.
-        So we return the original path, and indicate if this is
-        properly normalized.
-
-        :return: (path, is_normalized) Return a path which can
-                access the file, and whether or not this path is
-                normalized.
-        """
-        return path, unicodedata.normalize('NFKC', path) == path
+    normalized_filename = _inaccessible_normalized_filename
 
 
 def terminal_width():

=== modified file 'bzrlib/progress.py'
--- bzrlib/progress.py	2006-06-21 02:33:40 +0000
+++ bzrlib/progress.py	2006-07-10 23:00:44 +0000
@@ -39,17 +39,16 @@
 import sys
 import time
 import os
-from collections import deque
-
 
 import bzrlib.errors as errors
-from bzrlib.trace import mutter 
+from bzrlib.trace import mutter
 
 
 def _supports_progress(f):
-    if not hasattr(f, 'isatty'):
+    isatty = getattr(f, 'isatty', None)
+    if isatty is None:
         return False
-    if not f.isatty():
+    if not isatty():
         return False
     if os.environ.get('TERM') == 'dumb':
         # e.g. emacs compile window
@@ -57,16 +56,31 @@
     return True
 
 
+_progress_bar_types = {}
+
 
 def ProgressBar(to_file=None, **kwargs):
     """Abstract factory"""
     if to_file is None:
         to_file = sys.stderr
-    if _supports_progress(to_file):
-        return TTYProgressBar(to_file=to_file, **kwargs)
+    requested_bar_type = os.environ.get('BZR_PROGRESS_BAR')
+    # An value of '' or not set reverts to standard processing
+    if requested_bar_type in (None, ''):
+        if _supports_progress(to_file):
+            return TTYProgressBar(to_file=to_file, **kwargs)
+        else:
+            return DotsProgressBar(to_file=to_file, **kwargs)
     else:
-        return DotsProgressBar(to_file=to_file, **kwargs)
-    
+        # Minor sanitation to prevent spurious errors
+        requested_bar_type = requested_bar_type.lower().strip()
+        # TODO: jam 20060710 Arguably we shouldn't raise an exception
+        #       but should instead just disable progress bars if we
+        #       don't recognize the type
+        if requested_bar_type not in _progress_bar_types:
+            raise errors.InvalidProgressBarType(requested_bar_type,
+                                                _progress_bar_types.keys())
+        return _progress_bar_types[requested_bar_type](to_file=to_file, **kwargs)
+
  
 class ProgressBarStack(object):
     """A stack of progress bars."""
@@ -93,7 +107,7 @@
         self._show_count = show_count
         self._to_messages_file = to_messages_file
         self._stack = []
-        self._klass = klass or TTYProgressBar
+        self._klass = klass or ProgressBar
 
     def top(self):
         if len(self._stack) != 0:
@@ -206,6 +220,10 @@
         return DummyProgress(**kwargs)
 
 
+_progress_bar_types['dummy'] = DummyProgress
+_progress_bar_types['none'] = DummyProgress
+
+
 class DotsProgressBar(_BaseProgressBar):
 
     def __init__(self, **kwargs):
@@ -233,6 +251,9 @@
     def child_update(self, message, current, total):
         self.tick()
 
+
+_progress_bar_types['dots'] = DotsProgressBar
+
     
 class TTYProgressBar(_BaseProgressBar):
     """Progress bar display object.
@@ -262,8 +283,10 @@
         _BaseProgressBar.__init__(self, **kwargs)
         self.spin_pos = 0
         self.width = terminal_width()
-        self.last_updates = deque()
+        self.last_updates = []
+        self._max_last_updates = 10
         self.child_fraction = 0
+        self._have_output = False
     
 
     def throttle(self, old_msg):
@@ -284,6 +307,8 @@
             return True
 
         self.last_updates.append(now - self.last_update)
+        # Don't let the queue grow without bound
+        self.last_updates = self.last_updates[-self._max_last_updates:]
         self.last_update = now
         return False
         
@@ -405,13 +430,19 @@
 
         assert len(m) < self.width
         self.to_file.write('\r' + m.ljust(self.width - 1))
+        self._have_output = True
         #self.to_file.flush()
             
     def clear(self):        
-        self.to_file.write('\r%s\r' % (' ' * (self.width - 1)))
+        if self._have_output:
+            self.to_file.write('\r%s\r' % (' ' * (self.width - 1)))
+        self._have_output = False
         #self.to_file.flush()        
 
 
+_progress_bar_types['tty'] = TTYProgressBar
+
+
 class ChildProgress(_BaseProgressBar):
     """A progress indicator that pushes its data to the parent"""
 
@@ -487,8 +518,6 @@
     assert total_duration >= elapsed
 
     if last_updates and len(last_updates) >= n_recent:
-        while len(last_updates) > n_recent:
-            last_updates.popleft()
         avg = sum(last_updates) / float(len(last_updates))
         time_left = avg * (total - current)
 

=== modified file 'bzrlib/revision.py'
--- bzrlib/revision.py	2006-07-04 17:23:00 +0000
+++ bzrlib/revision.py	2006-07-08 19:32:30 +0000
@@ -75,8 +75,7 @@
         return not self.__eq__(other)
 
     def _check_properties(self):
-        """Verify that all revision properties are OK.
-        """
+        """Verify that all revision properties are OK."""
         for name, value in self.properties.iteritems():
             if not isinstance(name, basestring) or contains_whitespace(name):
                 raise ValueError("invalid property name %r" % name)

=== modified file 'bzrlib/revisionspec.py'
--- bzrlib/revisionspec.py	2006-05-24 00:06:33 +0000
+++ bzrlib/revisionspec.py	2006-07-08 19:32:30 +0000
@@ -149,7 +149,10 @@
             raise NoSuchRevision(branch, str(self.spec))
 
     def in_history(self, branch):
-        revs = branch.revision_history()
+        if branch:
+            revs = branch.revision_history()
+        else:
+            revs = None
         return self._match_on_and_check(branch, revs)
 
         # FIXME: in_history is somewhat broken,
@@ -190,10 +193,21 @@
 
     def _match_on(self, branch, revs):
         """Lookup a revision by revision number"""
-        try:
-            return RevisionInfo(branch, int(self.spec))
-        except ValueError:
-            return RevisionInfo(branch, None)
+        if self.spec.find(':') == -1:
+            try:
+                return RevisionInfo(branch, int(self.spec))
+            except ValueError:
+                return RevisionInfo(branch, None)
+        else:
+            from branch import Branch
+            revname = self.spec[self.spec.find(':')+1:]
+            other_branch = Branch.open_containing(revname)[0]
+            try:
+                revno = int(self.spec[:self.spec.find(':')])
+            except ValueError:
+                return RevisionInfo(other_branch, None)
+            revid = other_branch.get_rev_id(revno)
+            return RevisionInfo(other_branch, revno)
 
 SPEC_TYPES.append(RevisionSpec_revno)
 

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2006-07-04 15:04:11 +0000
+++ bzrlib/tests/blackbox/__init__.py	2006-07-09 20:12:49 +0000
@@ -64,6 +64,7 @@
                      'bzrlib.tests.blackbox.test_logformats',
                      'bzrlib.tests.blackbox.test_merge',
                      'bzrlib.tests.blackbox.test_missing',
+                     'bzrlib.tests.blackbox.test_mv',
                      'bzrlib.tests.blackbox.test_outside_wt',
                      'bzrlib.tests.blackbox.test_pull',
                      'bzrlib.tests.blackbox.test_push',

=== modified file 'bzrlib/tests/blackbox/test_commit.py'
--- bzrlib/tests/blackbox/test_commit.py	2006-06-23 16:26:26 +0000
+++ bzrlib/tests/blackbox/test_commit.py	2006-07-11 15:04:57 +0000
@@ -287,3 +287,16 @@
         # version or the u2 version.
         self.build_tree_contents([('u1/hosts', 'merge resolution\n')])
         self.run_bzr('commit', '-m', 'checkin merge of the offline work from u1', 'u1')
+
+    def test_commit_respects_spec_for_removals(self):
+        """Commit with a file spec should only commit removals that match"""
+        t = self.make_branch_and_tree('.')
+        self.build_tree(['file-a', 'dir-a/', 'dir-a/file-b'])
+        t.add(['file-a', 'dir-a', 'dir-a/file-b'])
+        t.commit('Create')
+        t.remove(['file-a', 'dir-a/file-b'])
+        os.chdir('dir-a')
+        result = self.run_bzr('commit', '.', '-m' 'removed file-b')[1]
+        self.assertNotContainsRe(result, 'file-a')
+        result = self.run_bzr('status')[0]
+        self.assertContainsRe(result, 'removed:\n  file-a')

=== modified file 'bzrlib/tests/blackbox/test_diff.py'
--- bzrlib/tests/blackbox/test_diff.py	2006-07-05 05:03:34 +0000
+++ bzrlib/tests/blackbox/test_diff.py	2006-07-08 20:38:36 +0000
@@ -157,6 +157,26 @@
                               "+contents of branch1/file\n"
                               "\n", subst_dates(out))
 
+    def test_diff_revno_branches(self):
+        self.example_branches()
+        print >> open('branch2/file', 'wb'), 'even newer content'
+        self.run_bzr_captured(['commit', '-m', 'update file once more', 'branch2'])
+
+        out, err = self.run_bzr_captured(['diff', '-r', 'revno:1:branch2..revno:1:branch1'],
+                                         retcode=0)
+        self.assertEquals('', err)
+        self.assertEquals('', out)
+        out, ett = self.run_bzr_captured(['diff', '-r', 'revno:2:branch2..revno:1:branch1'],
+                                         retcode=1)
+        self.assertEquals('', err)
+        self.assertEqualDiff("=== modified file 'file'\n"
+                              "--- file\tYYYY-MM-DD HH:MM:SS +ZZZZ\n"
+                              "+++ file\tYYYY-MM-DD HH:MM:SS +ZZZZ\n"
+                              "@@ -1,1 +1,1 @@\n"
+                              "-new content\n"
+                              "+contents of branch1/file\n"
+                              "\n", subst_dates(out))
+
     def example_branch2(self):
         self.build_tree(['branch1/', 'branch1/file1'], line_endings='binary')
         self.capture('init branch1')

=== added file 'bzrlib/tests/blackbox/test_mv.py'
--- bzrlib/tests/blackbox/test_mv.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_mv.py	2006-07-11 01:14:17 +0000
@@ -0,0 +1,130 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test for 'bzr mv'"""
+
+import os
+
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestMove(TestCaseWithTransport):
+
+    def test_mv_modes(self):
+        """Test two modes of operation for mv"""
+        tree = self.make_branch_and_tree('.')
+        files = self.build_tree(['a', 'c', 'subdir/'])
+        tree.add(['a', 'c', 'subdir'])
+
+        self.run_bzr('mv', 'a', 'b')
+        self.failUnlessExists('b')
+        self.failIfExists('a')
+
+        self.run_bzr('mv', 'b', 'subdir')
+        self.failUnlessExists('subdir/b')
+        self.failIfExists('b')
+
+        self.run_bzr('mv', 'subdir/b', 'a')
+        self.failUnlessExists('a')
+        self.failIfExists('subdir/b')
+
+        self.run_bzr('mv', 'a', 'c', 'subdir')
+        self.failUnlessExists('subdir/a')
+        self.failUnlessExists('subdir/c')
+        self.failIfExists('a')
+        self.failIfExists('c')
+
+        self.run_bzr('mv', 'subdir/a', 'subdir/newa')
+        self.failUnlessExists('subdir/newa')
+        self.failIfExists('subdir/a')
+
+    def test_mv_unversioned(self):
+        self.build_tree(['unversioned.txt'])
+        self.run_bzr_error(
+            ["^bzr: ERROR: can't rename: old name .* is not versioned$"],
+            'mv', 'unversioned.txt', 'elsewhere')
+
+    def test_mv_nonexisting(self):
+        self.run_bzr_error(
+            ["^bzr: ERROR: can't rename: old working file .* does not exist$"],
+            'mv', 'doesnotexist', 'somewhereelse')
+
+    def test_mv_unqualified(self):
+        self.run_bzr_error(['^bzr: ERROR: missing file argument$'], 'mv')
+        
+    def test_mv_invalid(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['test.txt', 'sub1/'])
+        tree.add(['test.txt'])
+
+        self.run_bzr_error(
+            ["^bzr: ERROR: destination u'sub1' is not a versioned directory$"],
+            'mv', 'test.txt', 'sub1')
+        
+        self.run_bzr_error(
+            ["^bzr: ERROR: can't determine destination directory id for u'sub1'$"],
+            'mv', 'test.txt', 'sub1/hello.txt')
+        
+    def test_mv_dirs(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['hello.txt', 'sub1/'])
+        tree.add(['hello.txt', 'sub1'])
+
+        self.run_bzr('mv', 'sub1', 'sub2')
+        self.failUnlessExists('sub2')
+        self.failIfExists('sub1')
+        self.run_bzr('mv', 'hello.txt', 'sub2')
+        self.failUnlessExists("sub2/hello.txt")
+        self.failIfExists("hello.txt")
+
+        tree.read_working_inventory()
+        tree.commit('commit with some things moved to subdirs')
+
+        self.build_tree(['sub1/'])
+        tree.add(['sub1'])
+        self.run_bzr('mv', 'sub2/hello.txt', 'sub1')
+        self.failIfExists('sub2/hello.txt')
+        self.failUnlessExists('sub1/hello.txt')
+        self.run_bzr('mv', 'sub2', 'sub1')
+        self.failIfExists('sub2')
+        self.failUnlessExists('sub1/sub2')
+
+    def test_mv_relative(self):
+        self.build_tree(['sub1/', 'sub1/sub2/', 'sub1/hello.txt'])
+        tree = self.make_branch_and_tree('.')
+        tree.add(['sub1', 'sub1/sub2', 'sub1/hello.txt'])
+        tree.commit('initial tree')
+
+        os.chdir('sub1/sub2')
+        self.run_bzr('mv', '../hello.txt', '.')
+        self.failUnlessExists('./hello.txt')
+        tree.read_working_inventory()
+        tree.commit('move to parent directory')
+
+        os.chdir('..')
+
+        self.run_bzr('mv', 'sub2/hello.txt', '.')
+        self.failUnlessExists('hello.txt')
+
+    def test_mv_smoke_aliases(self):
+        # just test that aliases for mv exist, if their behaviour is changed in
+        # the future, then extend the tests.
+        self.build_tree(['a'])
+        tree = self.make_branch_and_tree('.')
+        tree.add(['a'])
+
+        self.run_bzr('move', 'a', 'b')
+        self.run_bzr('rename', 'b', 'a')

=== modified file 'bzrlib/tests/blackbox/test_non_ascii.py'
--- bzrlib/tests/blackbox/test_non_ascii.py	2006-07-03 17:22:41 +0000
+++ bzrlib/tests/blackbox/test_non_ascii.py	2006-07-11 15:05:36 +0000
@@ -447,11 +447,11 @@
         bzr = self.run_bzr_decode
 
         txt = bzr('ls')
-        self.assertEqual(['a', 'b', self.info['filename']],
-                         txt.splitlines())
+        self.assertEqual(sorted(['a', 'b', self.info['filename']]),
+                         sorted(txt.splitlines()))
         txt = bzr('ls', '--null')
-        self.assertEqual(['a', 'b', self.info['filename'], ''],
-                         txt.split('\0'))
+        self.assertEqual(sorted(['', 'a', 'b', self.info['filename']]),
+                         sorted(txt.split('\0')))
 
         txt = bzr('ls', encoding='ascii', retcode=3)
         txt = bzr('ls', '--null', encoding='ascii', retcode=3)

=== modified file 'bzrlib/tests/blackbox/test_too_much.py'
--- bzrlib/tests/blackbox/test_too_much.py	2006-07-04 15:04:11 +0000
+++ bzrlib/tests/blackbox/test_too_much.py	2006-07-09 20:12:49 +0000
@@ -147,17 +147,6 @@
         self.runbzr('revert')
         os.chdir('..')
 
-    def test_mv_modes(self):
-        """Test two modes of operation for mv"""
-        self.runbzr('init')
-        self.build_tree(['a', 'c', 'subdir/'])
-        self.run_bzr_captured(['add', self.test_dir])
-        self.run_bzr_captured(['mv', 'a', 'b'])
-        self.run_bzr_captured(['mv', 'b', 'subdir'])
-        self.run_bzr_captured(['mv', 'subdir/b', 'a'])
-        self.run_bzr_captured(['mv', 'a', 'c', 'subdir'])
-        self.run_bzr_captured(['mv', 'subdir/a', 'subdir/newa'])
-
     def test_main_version(self):
         """Check output from version command and master option is reasonable"""
         # output is intentionally passed through to stdout so that we
@@ -677,81 +666,20 @@
         out = capture("help ci")
         out.index('aliases: ')
 
-        progress("can't rename unversioned file")
-        runbzr("rename test.txt new-test.txt", 3)
-
-        progress("adding a file")
-
-        runbzr("add test.txt")
-        self.assertEquals(capture("unknowns"), '')
-
-        progress("rename newly-added file")
-        runbzr("rename test.txt hello.txt")
-        self.assert_(os.path.exists("hello.txt"))
-        self.assert_(not os.path.exists("test.txt"))
-
-        self.assertEquals(capture("revno"), '0\n')
-
-        progress("add first revision")
-        runbzr(['commit', '-m', 'add first revision'])
-
-        progress("more complex renames")
-        os.mkdir("sub1")
-        runbzr("rename hello.txt sub1", 3)
-        runbzr("rename hello.txt sub1/hello.txt", 3)
-        runbzr("move hello.txt sub1", 3)
-
-        runbzr("add sub1")
-        runbzr("rename sub1 sub2")
-        runbzr("move hello.txt sub2")
-        self.assertEqual(capture("relpath sub2/hello.txt"),
-                         pathjoin("sub2", "hello.txt\n"))
-
-        self.assert_(exists("sub2"))
-        self.assert_(exists("sub2/hello.txt"))
-        self.assert_(not exists("sub1"))
-        self.assert_(not exists("hello.txt"))
-
-        runbzr(['commit', '-m', 'commit with some things moved to subdirs'])
-
-        mkdir("sub1")
-        runbzr('add sub1')
-        runbzr('move sub2/hello.txt sub1')
-        self.assert_(not exists('sub2/hello.txt'))
-        self.assert_(exists('sub1/hello.txt'))
-        runbzr('move sub2 sub1')
-        self.assert_(not exists('sub2'))
-        self.assert_(exists('sub1/sub2'))
-
-        runbzr(['commit', '-m', 'rename nested subdirectories'])
-
-        chdir('sub1/sub2')
-        self.assertEquals(capture('root')[:-1],
-                          pathjoin(self.test_dir, 'branch1'))
-        runbzr('move ../hello.txt .')
-        self.assert_(exists('./hello.txt'))
-        self.assertEquals(capture('relpath hello.txt'),
-                          pathjoin('sub1', 'sub2', 'hello.txt') + '\n')
-        self.assertEquals(capture('relpath ../../sub1/sub2/hello.txt'), pathjoin('sub1', 'sub2', 'hello.txt\n'))
-        runbzr(['commit', '-m', 'move to parent directory'])
-        chdir('..')
-        self.assertEquals(capture('relpath sub2/hello.txt'), pathjoin('sub1', 'sub2', 'hello.txt\n'))
-
-        runbzr('move sub2/hello.txt .')
-        self.assert_(exists('hello.txt'))
-
         f = file('hello.txt', 'wt')
         f.write('some nice new content\n')
         f.close()
 
+        runbzr("add hello.txt")
+        
         f = file('msg.tmp', 'wt')
         f.write('this is my new commit\nand it has multiple lines, for fun')
         f.close()
 
         runbzr('commit -F msg.tmp')
 
-        self.assertEquals(capture('revno'), '5\n')
-        runbzr('export -r 5 export-5.tmp')
+        self.assertEquals(capture('revno'), '1\n')
+        runbzr('export -r 1 export-1.tmp')
         runbzr('export export.tmp')
 
         runbzr('log')

=== modified file 'bzrlib/tests/test_commit.py'
--- bzrlib/tests/test_commit.py	2006-05-12 08:29:23 +0000
+++ bzrlib/tests/test_commit.py	2006-07-11 15:04:57 +0000
@@ -507,3 +507,16 @@
             ('deleted', 'filetoremove'),
             ],
             reporter.calls)
+
+    def test_commit_removals_respects_filespec(self):
+        """Commit respects the specified_files for removals."""
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a', 'b'])
+        tree.add(['a', 'b'])
+        tree.commit('added a, b')
+        tree.remove(['a', 'b'])
+        Commit().commit(message='removed a', working_tree=tree, 
+                        specific_files='a')
+        basis = tree.basis_tree().inventory
+        self.assertIs(None, basis.path2id('a'))
+        self.assertFalse(basis.path2id('b') is None)

=== modified file 'bzrlib/tests/test_hashcache.py'
--- bzrlib/tests/test_hashcache.py	2006-03-31 00:43:21 +0000
+++ bzrlib/tests/test_hashcache.py	2006-07-12 05:22:48 +0000
@@ -1,4 +1,4 @@
-# (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2006 Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,12 +16,13 @@
 
 import os
 import sha
+import stat
 import sys
 import time
 
 from bzrlib.errors import BzrError
 from bzrlib.hashcache import HashCache
-from bzrlib.tests import TestCaseInTempDir, TestSkipped
+from bzrlib.tests import TestCaseInTempDir, TestSkipped, TestCase
 
 
 def sha1(t):
@@ -29,117 +30,180 @@
 
 
 def pause():
-    if False:
-        return
-    if sys.platform in ('win32', 'cygwin'):
-        time.sleep(3)
-        return
-    # allow it to stabilize
-    start = int(time.time())
-    while int(time.time()) == start:
-        time.sleep(0.2)
-
-
-class FixThisError(Exception):
-    pass
-    
+    time.sleep(5.0)
+
 
 class TestHashCache(TestCaseInTempDir):
-
-    def test_hashcache(self):
-        """Functional tests for hashcache"""
-
+    """Test the hashcache against a real directory"""
+
+    def make_hashcache(self):
         # make a dummy bzr directory just to hold the cache
         os.mkdir('.bzr')
         hc = HashCache('.', '.bzr/stat-cache')
-
-        file('foo', 'wb').write('hello')
-        os.mkdir('subdir')
-        pause()
-
+        return hc
+
+    def reopen_hashcache(self):
+        hc = HashCache('.', '.bzr/stat-cache')
+        hc.read()
+        return hc
+
+    def test_hashcache_initial_miss(self):
+        """Get correct hash from an empty hashcache"""
+        hc = self.make_hashcache()
+        self.build_tree_contents([('foo', 'hello')])
         self.assertEquals(hc.get_sha1('foo'),
                           'aaf4c61ddcc5e8a2dabede0f3b482cd9aea9434d')
         self.assertEquals(hc.miss_count, 1)
         self.assertEquals(hc.hit_count, 0)
 
-        # check we hit without re-reading
-        self.assertEquals(hc.get_sha1('foo'),
-                          'aaf4c61ddcc5e8a2dabede0f3b482cd9aea9434d')
-        self.assertEquals(hc.miss_count, 1)
-        self.assertEquals(hc.hit_count, 1)
-
-        # check again without re-reading
-        self.assertEquals(hc.get_sha1('foo'),
-                          'aaf4c61ddcc5e8a2dabede0f3b482cd9aea9434d')
-        self.assertEquals(hc.miss_count, 1)
-        self.assertEquals(hc.hit_count, 2)
-
-        # write new file and make sure it is seen
-        file('foo', 'wb').write('goodbye')
-        pause()
-        self.assertEquals(hc.get_sha1('foo'),
-                          '3c8ec4874488f6090a157b014ce3397ca8e06d4f')
-        self.assertEquals(hc.miss_count, 2)
-
-        # quickly write new file of same size and make sure it is seen
-        # this may rely on detection of timestamps that are too close
-        # together to be safe
-        file('foo', 'wb').write('g00dbye')
-        self.assertEquals(hc.get_sha1('foo'),
-                          sha1('g00dbye'))
-
-        file('foo2', 'wb').write('other file')
-        self.assertEquals(hc.get_sha1('foo2'), sha1('other file'))
-
-        os.remove('foo2')
-        self.assertEquals(hc.get_sha1('foo2'), None)
-
-        file('foo2', 'wb').write('new content')
-        self.assertEquals(hc.get_sha1('foo2'), sha1('new content'))
-
+    def test_hashcache_new_file(self):
+        hc = self.make_hashcache()
+        self.build_tree_contents([('foo', 'goodbye')])
+        # now read without pausing; it may not be possible to cache it as its
+        # so new
+        self.assertEquals(hc.get_sha1('foo'), sha1('goodbye'))
+
+    def test_hashcache_nonexistent_file(self):
+        hc = self.make_hashcache()
+        self.assertEquals(hc.get_sha1('no-name-yet'), None)
+
+    def test_hashcache_replaced_file(self):
+        hc = self.make_hashcache()
+        self.build_tree_contents([('foo', 'goodbye')])
+        self.assertEquals(hc.get_sha1('foo'), sha1('goodbye'))
+        os.remove('foo')
+        self.assertEquals(hc.get_sha1('foo'), None)
+        self.build_tree_contents([('foo', 'new content')])
+        self.assertEquals(hc.get_sha1('foo'), sha1('new content'))
+
+    def test_hashcache_not_file(self):
+        hc = self.make_hashcache()
+        self.build_tree(['subdir/'])
         self.assertEquals(hc.get_sha1('subdir'), None)
 
-        # pause briefly to make sure they're not treated as new uncacheable
-        # files
+    def test_hashcache_load(self):
+        hc = self.make_hashcache()
+        self.build_tree_contents([('foo', 'contents')])
         pause()
-
-        self.assertEquals(hc.get_sha1('foo'), sha1('g00dbye'))
-        self.assertEquals(hc.get_sha1('foo2'), sha1('new content'))
-
-        # write out, read back in and check that we don't need to
-        # re-read any files
+        self.assertEquals(hc.get_sha1('foo'), sha1('contents'))
         hc.write()
-        del hc
-
-        hc = HashCache('.', '.bzr/stat-cache')
-        hc.read()
-
-        self.assertEquals(len(hc._cache), 2)
-        self.assertEquals(hc.get_sha1('foo'), sha1('g00dbye'))
+        hc = self.reopen_hashcache()
+        self.assertEquals(hc.get_sha1('foo'), sha1('contents'))
         self.assertEquals(hc.hit_count, 1)
-        self.assertEquals(hc.miss_count, 0)
-        self.assertEquals(hc.get_sha1('foo2'), sha1('new content'))
+
+    def test_hammer_hashcache(self):
+        hc = self.make_hashcache()
+        for i in xrange(10000):
+            self.log('start writing at %s', time.time())
+            f = file('foo', 'w')
+            try:
+                last_content = '%08x' % i
+                f.write(last_content)
+            finally:
+                f.close()
+            last_sha1 = sha1(last_content)
+            self.log("iteration %d: %r -> %r",
+                     i, last_content, last_sha1)
+            got_sha1 = hc.get_sha1('foo')
+            self.assertEquals(got_sha1, last_sha1)
+            hc.write()
+            hc = self.reopen_hashcache()
 
     def test_hashcache_raise(self):
         """check that hashcache can raise BzrError"""
-
-        os.mkdir('.bzr')
-        hc = HashCache('.', '.bzr/stat-cache')
-        ok = False
-
-        # make a best effort to create a weird kind of file
-        funcs = (getattr(os, 'mkfifo', None), getattr(os, 'mknod', None))
-        for func in funcs:
-            if func is None:
-                continue
-            try:
-                func('a')
-                ok = True
-                break
-            except FixThisError:
-                pass
-
-        if ok:
-            self.assertRaises(BzrError, hc.get_sha1, 'a')
+        hc = self.make_hashcache()
+        if not hasattr(os, 'mkfifo'):
+            raise TestSkipped('filesystem fifos not supported on this system')
+        os.mkfifo('a')
+        # It's possible that the system supports fifos but the filesystem
+        # can't.  In that case we should skip at this point.  But in fact
+        # such combinations don't usually occur for the filesystem where
+        # people test bzr.
+        self.assertRaises(BzrError, hc.get_sha1, 'a')
+
+
+class FakeHashCache(HashCache):
+    """Hashcache that consults a fake clock rather than the real one.
+
+    This lets us examine how old or new files would be handled, without
+    actually having to wait for time to pass.
+    """
+    def __init__(self):
+        # set root and cache file name to none to make sure we won't touch the
+        # real filesystem
+        HashCache.__init__(self, '.', 'hashcache')
+        self._files = {}
+        # simulated clock running forward as operations happen
+        self._clock = 0
+
+    def put_file(self, filename, file_contents):
+        abspath = './' + filename
+        self._files[abspath] = (file_contents, self._clock)
+
+    def _fingerprint(self, abspath):
+        entry = self._files[abspath]
+        return (len(entry[0]),
+                entry[1], entry[1],
+                10, 20,
+                stat.S_IFREG | 0600)
+
+    def _really_sha1_file(self, abspath):
+        if abspath in self._files:
+            return sha1(self._files[abspath][0])
         else:
-            raise TestSkipped('No weird file type could be created')
+            return None
+
+    def _cutoff_time(self):
+        return self._clock - 2
+
+    def pretend_to_sleep(self, secs):
+        self._clock += secs
+
+    
+class TestHashCacheFakeFilesystem(TestCaseInTempDir):
+    """Tests the hashcache using a simulated OS.
+    """
+
+    def make_hashcache(self):
+        return FakeHashCache()
+
+    def test_hashcache_miss_new_file(self):
+        """A new file gives the right sha1 but misses"""
+        hc = self.make_hashcache()
+        hc.put_file('foo', 'hello')
+        self.assertEquals(hc.get_sha1('foo'), sha1('hello'))
+        self.assertEquals(hc.miss_count, 1)
+        self.assertEquals(hc.hit_count, 0)
+        # if we try again it's still too new; 
+        self.assertEquals(hc.get_sha1('foo'), sha1('hello'))
+        self.assertEquals(hc.miss_count, 2)
+        self.assertEquals(hc.hit_count, 0)
+
+    def test_hashcache_old_file(self):
+        """An old file gives the right sha1 and hits"""
+        hc = self.make_hashcache()
+        hc.put_file('foo', 'hello')
+        hc.pretend_to_sleep(20)
+        # file is new; should get the correct hash but miss
+        self.assertEquals(hc.get_sha1('foo'), sha1('hello'))
+        self.assertEquals(hc.miss_count, 1)
+        self.assertEquals(hc.hit_count, 0)
+        # and can now be hit
+        self.assertEquals(hc.get_sha1('foo'), sha1('hello'))
+        self.assertEquals(hc.miss_count, 1)
+        self.assertEquals(hc.hit_count, 1)
+        hc.pretend_to_sleep(3)
+        # and again
+        self.assertEquals(hc.get_sha1('foo'), sha1('hello'))
+        self.assertEquals(hc.miss_count, 1)
+        self.assertEquals(hc.hit_count, 2)
+
+    def test_hashcache_invalidates(self):
+        hc = self.make_hashcache()
+        hc.put_file('foo', 'hello')
+        hc.pretend_to_sleep(20)
+        hc.get_sha1('foo')
+        hc.put_file('foo', 'h1llo')
+        self.assertEquals(hc.get_sha1('foo'), sha1('h1llo'))
+        self.assertEquals(hc.miss_count, 2)
+        self.assertEquals(hc.hit_count, 0)

=== modified file 'bzrlib/tests/test_inv.py'
--- bzrlib/tests/test_inv.py	2006-06-28 16:34:04 +0000
+++ bzrlib/tests/test_inv.py	2006-07-02 05:45:50 +0000
@@ -18,12 +18,11 @@
 import os
 import time
 
+from bzrlib import errors, inventory, osutils
 from bzrlib.branch import Branch
-import bzrlib.errors as errors
 from bzrlib.diff import internal_diff
 from bzrlib.inventory import (Inventory, ROOT_ID, InventoryFile,
     InventoryDirectory, InventoryEntry)
-import bzrlib.inventory as inventory
 from bzrlib.osutils import (has_symlinks, rename, pathjoin, is_inside_any, 
     is_inside_or_parent_of_any)
 from bzrlib.tests import TestCase, TestCaseWithTransport
@@ -200,6 +199,22 @@
         self.assertIsInstance(inventory.make_entry("directory", "name", ROOT_ID),
             inventory.InventoryDirectory)
 
+    def test_make_entry_non_normalized(self):
+        orig_normalized_filename = osutils.normalized_filename
+
+        try:
+            osutils.normalized_filename = osutils._accessible_normalized_filename
+            entry = inventory.make_entry("file", u'a\u030a', ROOT_ID)
+            self.assertEqual(u'\xe5', entry.name)
+            self.assertIsInstance(entry, inventory.InventoryFile)
+
+            osutils.normalized_filename = osutils._inaccessible_normalized_filename
+            self.assertRaises(errors.InvalidNormalization,
+                    inventory.make_entry, 'file', u'a\u030a', ROOT_ID)
+        finally:
+            osutils.normalized_filename = orig_normalized_filename
+
+
 class TestEntryDiffing(TestCaseWithTransport):
 
     def setUp(self):

=== modified file 'bzrlib/tests/test_nonascii.py'
--- bzrlib/tests/test_nonascii.py	2006-07-03 19:27:26 +0000
+++ bzrlib/tests/test_nonascii.py	2006-07-11 15:05:36 +0000
@@ -20,8 +20,9 @@
 import sys
 from unicodedata import normalize
 
-from bzrlib.osutils import pathjoin, normalizes_filenames, unicode_filename
-from bzrlib.tests import TestCaseWithTransport, TestSkipped
+from bzrlib import osutils
+from bzrlib.osutils import pathjoin
+from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
 from bzrlib.workingtree import WorkingTree
 
 
@@ -40,25 +41,59 @@
 
 
 a_circle_c = u'\xe5'
+a_circle_d = u'a\u030a'
 a_dots_c = u'\xe4'
-a_circle_d = u'a\u030a'
 a_dots_d = u'a\u0308'
 z_umlat_c = u'\u017d'
 z_umlat_d = u'Z\u030c'
 
 
-class UnicodeFilename(TestCaseWithTransport):
-    """Test that UnicodeFilename returns the expected values."""
+class TestNormalization(TestCase):
+    """Verify that we have our normalizations correct."""
 
-    def test_a_circle(self):
+    def test_normalize(self):
         self.assertEqual(a_circle_d, normalize('NFKD', a_circle_c))
         self.assertEqual(a_circle_c, normalize('NFKC', a_circle_d))
-
-        self.assertEqual((a_circle_c, True), unicode_filename(a_circle_c))
-        if normalizes_filenames():
-            self.assertEqual((a_circle_c, True), unicode_filename(a_circle_d))
+        self.assertEqual(a_dots_d, normalize('NFKD', a_dots_c))
+        self.assertEqual(a_dots_c, normalize('NFKC', a_dots_d))
+        self.assertEqual(z_umlat_d, normalize('NFKD', z_umlat_c))
+        self.assertEqual(z_umlat_c, normalize('NFKC', z_umlat_d))
+
+
+class NormalizedFilename(TestCaseWithTransport):
+    """Test normalized_filename and associated helpers"""
+
+    def test__accessible_normalized_filename(self):
+        anf = osutils._accessible_normalized_filename
+        # normalized_filename should allow plain ascii strings
+        # not just unicode strings
+        self.assertEqual((u'ascii', True), anf('ascii'))
+        self.assertEqual((a_circle_c, True), anf(a_circle_c))
+        self.assertEqual((a_circle_c, True), anf(a_circle_d))
+        self.assertEqual((a_dots_c, True), anf(a_dots_c))
+        self.assertEqual((a_dots_c, True), anf(a_dots_d))
+        self.assertEqual((z_umlat_c, True), anf(z_umlat_c))
+        self.assertEqual((z_umlat_c, True), anf(z_umlat_d))
+
+    def test__inaccessible_normalized_filename(self):
+        inf = osutils._inaccessible_normalized_filename
+        # normalized_filename should allow plain ascii strings
+        # not just unicode strings
+        self.assertEqual((u'ascii', True), inf('ascii'))
+        self.assertEqual((a_circle_c, True), inf(a_circle_c))
+        self.assertEqual((a_circle_c, False), inf(a_circle_d))
+        self.assertEqual((a_dots_c, True), inf(a_dots_c))
+        self.assertEqual((a_dots_c, False), inf(a_dots_d))
+        self.assertEqual((z_umlat_c, True), inf(z_umlat_c))
+        self.assertEqual((z_umlat_c, False), inf(z_umlat_d))
+
+    def test_functions(self):
+        if osutils.normalizes_filenames():
+            self.assertEqual(osutils.normalized_filename,
+                             osutils._accessible_normalized_filename)
         else:
-            self.assertEqual((a_circle_d, False), unicode_filename(a_circle_d))
+            self.assertEqual(osutils.normalized_filename,
+                             osutils._inaccessible_normalized_filename)
 
     def test_platform(self):
         # With FAT32 and certain encodings on win32
@@ -79,9 +114,9 @@
         present = sorted(os.listdir(u'.'))
         self.assertEqual(expected, present)
 
-    def test_access(self):
-        # We should always be able to access files by the path returned
-        # from unicode_filename
+    def test_access_normalized(self):
+        # We should always be able to access files created with 
+        # normalized filenames
         # With FAT32 and certain encodings on win32
         # a_circle_c and a_dots_c actually map to the same file
         # adding a suffix kicks in the 'preserving but insensitive'
@@ -93,9 +128,13 @@
             raise TestSkipped("filesystem cannot create unicode files")
 
         for fname in files:
-            path = unicode_filename(fname)[0]
             # We should get an exception if we can't open the file at
             # this location.
+            path, can_access = osutils.normalized_filename(fname)
+
+            self.assertEqual(path, fname)
+            self.assertTrue(can_access)
+
             f = open(path, 'rb')
             try:
                 # Check the contents
@@ -108,4 +147,32 @@
                              'contents of %s is incorrect: %r != %r'
                              % (path, shouldbe, actual))
 
-
+    def test_access_non_normalized(self):
+        # Sometimes we can access non-normalized files by their normalized
+        # path, verify that normalized_filename returns the right info
+        files = [a_circle_d+'.1', a_dots_d+'.2', z_umlat_d+'.3']
+
+        try:
+            self.build_tree(files)
+        except UnicodeError:
+            raise TestSkipped("filesystem cannot create unicode files")
+
+        for fname in files:
+            # We should get an exception if we can't open the file at
+            # this location.
+            path, can_access = osutils.normalized_filename(fname)
+
+            self.assertNotEqual(path, fname)
+
+            # We should always be able to access them from the name
+            # they were created with
+            f = open(fname, 'rb')
+            f.close()
+
+            # And normalized_filename sholud tell us correctly if we can
+            # access them by an alternate name
+            if can_access:
+                f = open(path, 'rb')
+                f.close()
+            else:
+                self.assertRaises(IOError, open, path, 'rb')

=== modified file 'bzrlib/tests/test_options.py'
--- bzrlib/tests/test_options.py	2006-06-20 03:30:14 +0000
+++ bzrlib/tests/test_options.py	2006-07-11 21:46:11 +0000
@@ -46,6 +46,10 @@
         out, err = self.run_bzr_captured(['help', '-r'], retcode=3)
         self.assertContainsRe(err, r'unknown option')
 
+    def test_allow_dash(self):
+        """Test that we can pass a plain '-' as an argument."""
+        self.assertEqual((['-'], {}), parse_args(cmd_commit(), ['-']))
+
 
 #     >>> parse_args('log -r 500'.split())
 #     (['log'], {'revision': [<RevisionSpec_int 500>]})

=== modified file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	2006-06-29 20:26:43 +0000
+++ bzrlib/tests/test_osutils.py	2006-07-03 16:19:30 +0000
@@ -198,16 +198,16 @@
     def test_getcwd(self):
         # Make sure getcwd can handle unicode filenames
         try:
-            os.mkdir(u'B\xe5gfors')
+            os.mkdir(u'mu-\xb5')
         except UnicodeError:
             raise TestSkipped("Unable to create Unicode filename")
 
-        os.chdir(u'B\xe5gfors')
+        os.chdir(u'mu-\xb5')
         # TODO: jam 20060427 This will probably fail on Mac OSX because
         #       it will change the normalization of B\xe5gfors
         #       Consider using a different unicode character, or make
         #       osutils.getcwd() renormalize the path.
-        self.assertTrue(osutils._win32_getcwd().endswith(u'/B\xe5gfors'))
+        self.assertEndsWith(osutils._win32_getcwd(), u'mu-\xb5')
 
     def test_mkdtemp(self):
         tmpdir = osutils._win32_mkdtemp(dir='.')
@@ -257,6 +257,30 @@
             self.assertEqual(errno.ENOENT, e.errno)
 
 
+class TestMacFuncsDirs(TestCaseInTempDir):
+    """Test mac special functions that require directories."""
+
+    def test_getcwd(self):
+        # On Mac, this will actually create Ba\u030agfors
+        # but chdir will still work, because it accepts both paths
+        try:
+            os.mkdir(u'B\xe5gfors')
+        except UnicodeError:
+            raise TestSkipped("Unable to create Unicode filename")
+
+        os.chdir(u'B\xe5gfors')
+        self.assertEndsWith(osutils._mac_getcwd(), u'B\xe5gfors')
+
+    def test_getcwd_nonnorm(self):
+        # Test that _mac_getcwd() will normalize this path
+        try:
+            os.mkdir(u'Ba\u030agfors')
+        except UnicodeError:
+            raise TestSkipped("Unable to create Unicode filename")
+
+        os.chdir(u'Ba\u030agfors')
+        self.assertEndsWith(osutils._mac_getcwd(), u'B\xe5gfors')
+
 class TestSplitLines(TestCase):
 
     def test_split_unicode(self):

=== modified file 'bzrlib/tests/test_progress.py'
--- bzrlib/tests/test_progress.py	2006-06-21 03:14:59 +0000
+++ bzrlib/tests/test_progress.py	2006-07-10 23:00:44 +0000
@@ -14,8 +14,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+import os
 from StringIO import StringIO
 
+from bzrlib import errors
 from bzrlib.progress import (
         DummyProgress, ChildProgress,
         TTYProgressBar,
@@ -24,6 +26,7 @@
         )
 from bzrlib.tests import TestCase
 
+
 class FakeStack:
     def __init__(self, top):
         self.__top = top
@@ -44,6 +47,20 @@
             self.always_throttled = False
         
 
+class _TTYStringIO(StringIO):
+    """A helper class which makes a StringIO look like a terminal"""
+
+    def isatty(self):
+        return True
+
+
+class _NonTTYStringIO(StringIO):
+    """Helper that implements isatty() but returns False"""
+
+    def isatty(self):
+        return False
+
+
 class TestProgress(TestCase):
     def setUp(self):
         q = DummyProgress()
@@ -120,3 +137,174 @@
         # Updates after a second should not be squelched
         pb.update('me', 1, 1)
         self.assertFalse(pb.always_throttled)
+
+    def test_clear(self):
+        sio = StringIO()
+        pb = TTYProgressBar(to_file=sio, show_eta=False)
+        pb.width = 20 # Just make it easier to test
+        # This should not output anything
+        pb.clear()
+        # These two should not be displayed because
+        # of throttling
+        pb.update('foo', 1, 3)
+        pb.update('bar', 2, 3)
+        # So pb.clear() has nothing to do
+        pb.clear()
+
+        # Make sure the next update isn't throttled
+        pb.start_time -= 1
+        pb.update('baz', 3, 3)
+        pb.clear()
+
+        self.assertEqual('\r[=========] baz 3/3'
+                         '\r                   \r',
+                         sio.getvalue())
+
+    def test_no_eta(self):
+        # An old version of the progress bar would
+        # store every update if show_eta was false
+        # because the eta routine was where it was
+        # cleaned out
+        pb = InstrumentedProgress(to_file=StringIO(), show_eta=False)
+        # Just make sure this first few are throttled
+        pb.start_time += 5
+
+        # These messages are throttled, and don't contribute
+        for count in xrange(100):
+            pb.update('x', count, 300)
+        self.assertEqual(0, len(pb.last_updates))
+
+        # Unthrottle by time
+        pb.start_time -= 10
+
+        # These happen too fast, so only one gets through
+        for count in xrange(100):
+            pb.update('x', count+100, 200)
+        self.assertEqual(1, len(pb.last_updates))
+
+        pb.MIN_PAUSE = 0.0
+
+        # But all of these go through, don't let the
+        # last_update list grow without bound
+        for count in xrange(100):
+            pb.update('x', count+100, 200)
+
+        self.assertEqual(pb._max_last_updates, len(pb.last_updates))
+
+
+class TestProgressTypes(TestCase):
+    """Test that the right ProgressBar gets instantiated at the right time."""
+
+    def get_nested(self, outf, term, env_progress=None):
+        """Setup so that ProgressBar thinks we are in the supplied terminal."""
+        orig_term = os.environ.get('TERM')
+        orig_progress = os.environ.get('BZR_PROGRESS_BAR')
+        os.environ['TERM'] = term
+        if env_progress is not None:
+            os.environ['BZR_PROGRESS_BAR'] = env_progress
+        elif orig_progress is not None:
+            del os.environ['BZR_PROGRESS_BAR']
+
+        def reset():
+            if orig_term is None:
+                del os.environ['TERM']
+            else:
+                os.environ['TERM'] = orig_term
+            # We may have never created BZR_PROGRESS_BAR
+            # So we can't just delete like we can 'TERM' (which is always set)
+            if orig_progress is None:
+                if 'BZR_PROGRESS_BAR' in os.environ:
+                    del os.environ['BZR_PROGRESS_BAR']
+            else:
+                os.environ['BZR_PROGRESS_BAR'] = orig_progress
+
+        self.addCleanup(reset)
+
+        stack = ProgressBarStack(to_file=outf)
+        pb = stack.get_nested()
+        pb.start_time -= 1 # Make sure it is ready to write
+        pb.width = 20 # And it is of reasonable size
+        return pb
+
+    def test_tty_progress(self):
+        # Make sure the ProgressBarStack thinks it is
+        # writing out to a terminal, and thus uses a TTYProgressBar
+        out = _TTYStringIO()
+        pb = self.get_nested(out, 'xterm')
+        self.assertIsInstance(pb, TTYProgressBar)
+        try:
+            pb.update('foo', 1, 2)
+            pb.update('bar', 2, 2)
+        finally:
+            pb.finished()
+
+        self.assertEqual('\r/ [====   ] foo 1/2'
+                         '\r- [=======] bar 2/2'
+                         '\r                   \r',
+                         out.getvalue())
+
+    def test_dots_progress(self):
+        # Make sure the ProgressBarStack thinks it is
+        # not writing out to a terminal, and thus uses a 
+        # DotsProgressBar
+        out = _NonTTYStringIO()
+        pb = self.get_nested(out, 'xterm')
+        self.assertIsInstance(pb, DotsProgressBar)
+        try:
+            pb.update('foo', 1, 2)
+            pb.update('bar', 2, 2)
+        finally:
+            pb.finished()
+
+        self.assertEqual('foo: .'
+                         '\nbar: .'
+                         '\n',
+                         out.getvalue())
+
+    def test_no_isatty_progress(self):
+        # Make sure ProgressBarStack handles a plain StringIO()
+        import cStringIO
+        out = cStringIO.StringIO()
+        pb = self.get_nested(out, 'xterm')
+        pb.finished()
+        self.assertIsInstance(pb, DotsProgressBar)
+
+    def test_dumb_progress(self):
+        # Make sure the ProgressBarStack thinks it is writing out to a 
+        # terminal, but it is the emacs 'dumb' terminal, so it uses
+        # Dots
+        out = _TTYStringIO()
+        pb = self.get_nested(out, 'dumb')
+        pb.finished()
+        self.assertIsInstance(pb, DotsProgressBar)
+
+    def test_progress_env_tty(self):
+        # The environ variable BZR_PROGRESS_BAR controls what type of
+        # progress bar we will get, even if it wouldn't usually be that type
+        import cStringIO
+
+        # Usually, this would be a DotsProgressBar
+        out = cStringIO.StringIO()
+        pb = self.get_nested(out, 'dumb', 'tty')
+        pb.finished()
+        # Even though we are not a tty, the env_var will override
+        self.assertIsInstance(pb, TTYProgressBar)
+
+    def test_progress_env_dots(self):
+        # Even though we are in a tty, the env_var will override
+        out = _TTYStringIO()
+        pb = self.get_nested(out, 'xterm', 'dots')
+        pb.finished()
+        self.assertIsInstance(pb, DotsProgressBar)
+
+    def test_progress_env_none(self):
+        # Even though we are in a valid tty, no progress
+        out = _TTYStringIO()
+        pb = self.get_nested(out, 'xterm', 'none')
+        pb.finished()
+        self.assertIsInstance(pb, DummyProgress)
+
+    def test_progress_env_invalid(self):
+        out = _TTYStringIO()
+        self.assertRaises(errors.InvalidProgressBarType, self.get_nested,
+            out, 'xterm', 'nonexistant')

=== modified file 'bzrlib/tests/test_revisionnamespaces.py'
--- bzrlib/tests/test_revisionnamespaces.py	2006-05-24 00:06:33 +0000
+++ bzrlib/tests/test_revisionnamespaces.py	2006-07-08 20:38:36 +0000
@@ -27,6 +27,36 @@
 
 class TestRevisionNamespaces(TestCaseWithTransport):
 
+    def test_revno_n_path(self):
+        """Test revision specifiers.
+
+        These identify revisions by date, etc."""
+        wta = self.make_branch_and_tree('a')
+        ba = wta.branch
+        
+        wta.commit('Commit one', rev_id='a@r-0-1')
+        wta.commit('Commit two', rev_id='a@r-0-2')
+        wta.commit('Commit three', rev_id='a@r-0-3')
+
+        wtb = self.make_branch_and_tree('b')
+        bb = wtb.branch
+
+        wtb.commit('Commit one', rev_id='b@r-0-1')
+        wtb.commit('Commit two', rev_id='b@r-0-2')
+        wtb.commit('Commit three', rev_id='b@r-0-3')
+
+        self.assertEquals(RevisionSpec('revno:1:a/').in_history(ba),
+                          (1, 'a@r-0-1'))
+        # The argument of in_history should be ignored since it is
+        # redundant with the path in the spec.
+        self.assertEquals(RevisionSpec('revno:1:a/').in_history(None),
+                          (1, 'a@r-0-1'))
+        self.assertEquals(RevisionSpec('revno:1:a/').in_history(bb),
+                          (1, 'a@r-0-1'))
+        self.assertEquals(RevisionSpec('revno:2:b/').in_history(None),
+                          (2, 'b@r-0-2'))
+
+
     def test_revision_namespaces(self):
         """Test revision specifiers.
 

=== modified file 'bzrlib/tests/test_smart_add.py'
--- bzrlib/tests/test_smart_add.py	2006-07-11 17:12:14 +0000
+++ bzrlib/tests/test_smart_add.py	2006-07-12 14:24:06 +0000
@@ -1,8 +1,9 @@
 import os
 import unittest
 
+from bzrlib import errors, osutils
 from bzrlib.add import smart_add, smart_add_tree
-from bzrlib.tests import TestCaseWithTransport, TestCase
+from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
 from bzrlib.errors import NoSuchFile
 from bzrlib.inventory import InventoryFile, Inventory
 from bzrlib.workingtree import WorkingTree
@@ -212,6 +213,64 @@
                     'Accidentally added path: %s' % (path,))
 
 
+class TestAddNonNormalized(TestCaseWithTransport):
+
+    def make(self):
+        try:
+            self.build_tree([u'a\u030a'])
+        except UnicodeError:
+            raise TestSkipped('Filesystem cannot create unicode filenames')
+
+        self.wt = self.make_branch_and_tree('.')
+
+    def test_accessible_explicit(self):
+        self.make()
+        orig = osutils.normalized_filename
+        osutils.normalized_filename = osutils._accessible_normalized_filename
+        try:
+            smart_add_tree(self.wt, [u'a\u030a'])
+            self.assertEqual([(u'\xe5', 'file')],
+                    [(path, ie.kind) for path,ie in 
+                        self.wt.inventory.iter_entries()])
+        finally:
+            osutils.normalized_filename = orig
+
+    def test_accessible_implicit(self):
+        self.make()
+        orig = osutils.normalized_filename
+        osutils.normalized_filename = osutils._accessible_normalized_filename
+        try:
+            smart_add_tree(self.wt, [])
+            self.assertEqual([(u'\xe5', 'file')],
+                    [(path, ie.kind) for path,ie in 
+                        self.wt.inventory.iter_entries()])
+        finally:
+            osutils.normalized_filename = orig
+
+    def test_inaccessible_explicit(self):
+        self.make()
+        orig = osutils.normalized_filename
+        osutils.normalized_filename = osutils._inaccessible_normalized_filename
+        try:
+            self.assertRaises(errors.InvalidNormalization,
+                    smart_add_tree, self.wt, [u'a\u030a'])
+        finally:
+            osutils.normalized_filename = orig
+
+    def test_inaccessible_implicit(self):
+        self.make()
+        orig = osutils.normalized_filename
+        osutils.normalized_filename = osutils._inaccessible_normalized_filename
+        try:
+            # TODO: jam 20060701 In the future, this should probably
+            #       just ignore files that don't fit the normalization
+            #       rules, rather than exploding
+            self.assertRaises(errors.InvalidNormalization,
+                    smart_add_tree, self.wt, [])
+        finally:
+            osutils.normalized_filename = orig
+
+
 class TestAddActions(TestCase):
 
     def test_quiet(self):

=== modified file 'bzrlib/tests/test_ui.py'
--- bzrlib/tests/test_ui.py	2006-06-21 02:33:40 +0000
+++ bzrlib/tests/test_ui.py	2006-07-12 02:45:22 +0000
@@ -26,6 +26,7 @@
 import bzrlib.errors as errors
 from bzrlib.progress import TTYProgressBar, ProgressBarStack
 from bzrlib.tests import TestCase
+from bzrlib.tests.test_progress import _TTYStringIO
 from bzrlib.ui import SilentUIFactory
 from bzrlib.ui.text import TextUIFactory
 
@@ -63,18 +64,41 @@
     def test_progress_note(self):
         stderr = StringIO()
         stdout = StringIO()
-        ui_factory = TextUIFactory()
-        pb = ui_factory.nested_progress_bar()
-        try:
-            pb.to_messages_file = stdout
-            ui_factory._progress_bar_stack.bottom().to_file = stderr
+        ui_factory = TextUIFactory(bar_type=TTYProgressBar)
+        pb = ui_factory.nested_progress_bar()
+        try:
+            pb.to_messages_file = stdout
+            ui_factory._progress_bar_stack.bottom().to_file = stderr
+            result = pb.note('t')
+            self.assertEqual(None, result)
+            self.assertEqual("t\n", stdout.getvalue())
+            # Since there was no update() call, there should be no clear() call
+            self.failIf(re.search(r'^\r {10,}\r$', stderr.getvalue()) is not None,
+                        'We cleared the stderr without anything to put there')
+        finally:
+            pb.finished()
+
+    def test_progress_note_clears(self):
+        stderr = StringIO()
+        stdout = StringIO()
+        # The PQM redirects the output to a file, so it
+        # defaults to creating a Dots progress bar. we
+        # need to force it to believe we are a TTY
+        ui_factory = TextUIFactory(bar_type=TTYProgressBar)
+        pb = ui_factory.nested_progress_bar()
+        try:
+            pb.to_messages_file = stdout
+            ui_factory._progress_bar_stack.bottom().to_file = stderr
+            # Create a progress update that isn't throttled
+            pb.start_time -= 10
+            pb.update('x', 1, 1)
             result = pb.note('t')
             self.assertEqual(None, result)
             self.assertEqual("t\n", stdout.getvalue())
             # the exact contents will depend on the terminal width and we don't
             # care about that right now - but you're probably running it on at
             # least a 10-character wide terminal :)
-            self.assertContainsRe(stderr.getvalue(), r'^\r {10,}\r$')
+            self.assertContainsRe(stderr.getvalue(), r'\r {10,}\r$')
         finally:
             pb.finished()
 
@@ -153,7 +177,7 @@
     def test_text_factory_prompts_and_clears(self):
         # a get_boolean call should clear the pb before prompting
         factory = bzrlib.ui.text.TextUIFactory()
-        factory.stdout = StringIO()
+        factory.stdout = _TTYStringIO()
         factory.stdin = StringIO("yada\ny\n")
         pb = self.apply_redirected(
             factory.stdin, factory.stdout, factory.stdout, factory.nested_progress_bar)
@@ -174,4 +198,4 @@
             "\r   *" 
             "\rwhat do you want\\? \\[y/n\\]:what do you want\\? \\[y/n\\]:", 
             output):
-            self.fail("didn't match factory output %r" % factory)
+            self.fail("didn't match factory output %r, %s" % (factory, output))

=== modified file 'bzrlib/tests/test_urlutils.py'
--- bzrlib/tests/test_urlutils.py	2006-06-30 15:32:22 +0000
+++ bzrlib/tests/test_urlutils.py	2006-07-11 18:34:09 +0000
@@ -17,11 +17,12 @@
 """Tests for the urlutils wrapper."""
 
 import os
+import re
 import sys
 
+from bzrlib import osutils, urlutils
 import bzrlib
 from bzrlib.errors import InvalidURL, InvalidURLJoin
-import bzrlib.urlutils as urlutils
 from bzrlib.tests import TestCaseInTempDir, TestCase, TestSkipped
 
 
@@ -464,3 +465,30 @@
         test('http://host/', 'http://host', 'http://host/')
         #test('.', 'http://host/', 'http://host')
         test('http://host', 'http://host/', 'http://host')
+
+
+class TestCwdToURL(TestCaseInTempDir):
+    """Test that local_path_to_url works base on the cwd"""
+
+    def test_dot(self):
+        # This test will fail if getcwd is not ascii
+        os.mkdir('mytest')
+        os.chdir('mytest')
+
+        url = urlutils.local_path_to_url('.')
+        self.assertEndsWith(url, '/mytest')
+
+    def test_non_ascii(self):
+        try:
+            os.mkdir(u'dod\xe9')
+        except UnicodeError:
+            raise TestSkipped('cannot create unicode directory')
+
+        os.chdir(u'dod\xe9')
+
+        # On Mac OSX this directory is actually: 
+        #   u'/dode\u0301' => '/dode\xcc\x81
+        # but we should normalize it back to 
+        #   u'/dod\xe9' => '/dod\xc3\xa9'
+        url = urlutils.local_path_to_url('.')
+        self.assertEndsWith(url, '/dod%C3%A9')

=== modified file 'bzrlib/tests/workingtree_implementations/test_workingtree.py'
--- bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-07-07 15:22:42 +0000
+++ bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-07-12 10:09:27 +0000
@@ -20,7 +20,7 @@
 import sys
 
 import bzrlib
-from bzrlib import branch, bzrdir, errors, urlutils, workingtree
+from bzrlib import branch, bzrdir, errors, osutils, urlutils, workingtree
 from bzrlib.errors import (NotBranchError, NotVersionedError, 
                            UnsupportedOperation, PathsNotVersionedError)
 from bzrlib.osutils import pathjoin, getcwd, has_symlinks
@@ -182,20 +182,6 @@
         self.assertEquals(list(tree.unknowns()),
                           ['hello.txt'])
 
-    def test_hashcache(self):
-        from bzrlib.tests.test_hashcache import pause
-        tree = self.make_branch_and_tree('.')
-        self.build_tree(['hello.txt',
-                         'hello.txt.~1~'])
-        tree.add('hello.txt')
-        pause()
-        sha = tree.get_file_sha1(tree.path2id('hello.txt'))
-        self.assertEqual(1, tree._hashcache.miss_count)
-        tree2 = WorkingTree.open('.')
-        sha2 = tree2.get_file_sha1(tree2.path2id('hello.txt'))
-        self.assertEqual(0, tree2._hashcache.miss_count)
-        self.assertEqual(1, tree2._hashcache.hit_count)
-
     def test_initialize(self):
         # initialize should create a working tree and branch in an existing dir
         t = self.make_branch_and_tree('.')
@@ -632,4 +618,34 @@
         self.assertEqual((u'foo.pyc', 'V', 'file', 'anid'), files[1][:-1])
         self.assertEqual(2, len(files))
 
+    def test_non_normalized_add_accessible(self):
+        try:
+            self.build_tree([u'a\u030a'])
+        except UnicodeError:
+            raise TestSkipped('Filesystem does not support unicode filenames')
+        tree = self.make_branch_and_tree('.')
+        orig = osutils.normalized_filename
+        osutils.normalized_filename = osutils._accessible_normalized_filename
+        try:
+            tree.add([u'a\u030a'])
+            self.assertEqual([(u'\xe5', 'file')],
+                    [(path, ie.kind) for path,ie in 
+                                tree.inventory.iter_entries()])
+        finally:
+            osutils.normalized_filename = orig
+
+    def test_non_normalized_add_inaccessible(self):
+        try:
+            self.build_tree([u'a\u030a'])
+        except UnicodeError:
+            raise TestSkipped('Filesystem does not support unicode filenames')
+        tree = self.make_branch_and_tree('.')
+        orig = osutils.normalized_filename
+        osutils.normalized_filename = osutils._inaccessible_normalized_filename
+        try:
+            self.assertRaises(errors.InvalidNormalization,
+                tree.add, [u'a\u030a'])
+        finally:
+            osutils.normalized_filename = orig
+
 

=== modified file 'bzrlib/trace.py'
--- bzrlib/trace.py	2006-06-29 16:24:56 +0000
+++ bzrlib/trace.py	2006-07-12 02:49:48 +0000
@@ -256,6 +256,8 @@
 
 def report_exception(exc_info, err_file):
     exc_type, exc_object, exc_tb = exc_info
+    # Log the full traceback to ~/.bzr.log
+    log_exception_quietly()
     if (isinstance(exc_object, IOError)
         and getattr(exc_object, 'errno', None) == errno.EPIPE):
         print >>err_file, "bzr: broken pipe"

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-07-03 04:59:44 +0000
+++ bzrlib/workingtree.py	2006-07-03 18:27:35 +0000
@@ -51,11 +51,10 @@
 from time import time
 import warnings
 
+from bzrlib import bzrdir, errors, osutils, urlutils
 from bzrlib.atomicfile import AtomicFile
 from bzrlib.conflicts import Conflict, ConflictList, CONFLICT_SUFFIXES
-import bzrlib.bzrdir as bzrdir
 from bzrlib.decorators import needs_read_lock, needs_write_lock
-import bzrlib.errors as errors
 from bzrlib.errors import (BzrCheckError,
                            BzrError,
                            ConflictFormatError,
@@ -103,7 +102,6 @@
 from bzrlib.trace import mutter, note
 from bzrlib.transport import get_transport
 from bzrlib.transport.local import LocalTransport
-import bzrlib.urlutils as urlutils
 import bzrlib.ui
 import bzrlib.xml5
 
@@ -356,7 +354,7 @@
         :return: The WorkingTree that contains 'path', and the rest of path
         """
         if path is None:
-            path = os.getcwdu()
+            path = osutils.getcwd()
         control, relpath = bzrdir.BzrDir.open_containing(path)
 
         return control.open_workingtree(), relpath
@@ -768,7 +766,25 @@
                 elif self.is_ignored(fp[1:]):
                     c = 'I'
                 else:
-                    c = '?'
+                    # we may not have found this file, because of a unicode issue
+                    f_norm, can_access = osutils.normalized_filename(f)
+                    if f == f_norm or not can_access:
+                        # No change, so treat this file normally
+                        c = '?'
+                    else:
+                        # this file can be accessed by a normalized path
+                        # check again if it is versioned
+                        # these lines are repeated here for performance
+                        f = f_norm
+                        fp = from_dir_relpath + '/' + f
+                        fap = from_dir_abspath + '/' + f
+                        f_ie = inv.get_child(from_dir_id, f)
+                        if f_ie:
+                            c = 'V'
+                        elif self.is_ignored(fp[1:]):
+                            c = 'I'
+                        else:
+                            c = '?'
 
                 fk = file_kind(fap)
 
@@ -1000,9 +1016,15 @@
 
             fl = []
             for subf in os.listdir(dirabs):
-                if (subf != '.bzr'
-                    and (subf not in dir_entry.children)):
-                    fl.append(subf)
+                if subf == '.bzr':
+                    continue
+                if subf not in dir_entry.children:
+                    subf_norm, can_access = osutils.normalized_filename(subf)
+                    if subf_norm != subf and can_access:
+                        if subf_norm not in dir_entry.children:
+                            fl.append(subf_norm)
+                    else:
+                        fl.append(subf)
             
             fl.sort()
             for subf in fl:

